import { RSTOptions } from './opts';
import { Paragraph } from './dom';
export declare function quoteRST(text: string, escape_starting_whitespace?: boolean, escape_ending_whitespace?: boolean): string;
export declare function toRST(paragraphs: Paragraph[], opts?: RSTOptions): string;
//# sourceMappingURL=rst.d.ts.map