import { Connection } from "vscode-languageserver";
import { WorkspaceFolderContext } from "../services/workspaceManager";
export declare class ImagePuller {
    private connection;
    private context;
    private useProgressTracker;
    private _containerEngine;
    private _containerImage;
    private _pullPolicy;
    private _pullArguments;
    constructor(connection: Connection, context: WorkspaceFolderContext, containerEngine: string, containerImage: string, pullPolicy: string, pullArguments: string);
    setupImage(): Promise<boolean>;
    private determinePull;
    private checkForImage;
}
