"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isNullLiteral = void 0;
const utils_1 = require("@typescript-eslint/utils");
function isNullLiteral(i) {
    return i.type === utils_1.AST_NODE_TYPES.Literal && i.value == null;
}
exports.isNullLiteral = isNullLiteral;
//# sourceMappingURL=isNullLiteral.js.map