module.exports={A:{A:{"2":"J D E F A xB","548":"B"},B:{"1":"P Q R S T U V W Z a b c d e f g h i j k l H X m","516":"C K L G M N O"},C:{"1":"Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W Z a b c d e f g h i j k l H X m","2":"yB mB I n J D E F zB 0B","676":"0 1 2 3 4 5 6 7 8 9 A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB","1700":"GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB"},D:{"1":"bB cB dB eB fB gB hB iB P Q R S T U V W Z a b c d e f g h i j k l H X m 1B 2B 3B","2":"I n J D E F A B C K L","676":"G M N O o","804":"0 1 2 3 4 5 6 7 8 9 p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB"},E:{"2":"I n 4B qB","548":"tB uB lB CC","676":"5B","804":"J D E F A B C K L G 6B 7B 8B rB jB kB 9B AC BC sB"},F:{"1":"Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W kB","2":"F B C DC EC FC GC jB vB HC","804":"0 1 2 3 4 5 6 7 8 9 G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB"},G:{"2":"E qB IC wB JC KC LC MC NC OC PC QC RC SC","2052":"TC UC VC WC XC YC ZC aC bC sB tB uB lB"},H:{"2":"cC"},I:{"2":"mB I H dC eC fC gC wB hC iC"},J:{"2":"D","292":"A"},K:{"2":"A B C Y jB vB kB"},L:{"1":"X"},M:{"1":"H"},N:{"2":"A","548":"B"},O:{"804":"jC"},P:{"1":"rB pC qC rC sC tC lB uC","804":"I kC lC mC nC oC"},Q:{"804":"vC"},R:{"804":"wC"},S:{"1":"xC"}},B:1,C:"Full Screen API"};
