/**
Strip UTF-8 [byte order mark](https://en.wikipedia.org/wiki/Byte_order_mark#UTF-8) (BOM) from a string.

@example
```
import stripBom = require('strip-bom');

stripBom('\uFEFFunicorn');
//=> 'unicorn'
```
*/
declare function stripBom(string: string): string;

export = stripBom;
