import { Connection } from "vscode-languageserver";
import { WorkspaceFolderContext } from "./workspaceManager";
export declare class AnsibleConfig {
    private connection;
    private context;
    private _collection_paths;
    private _module_locations;
    private _ansible_location;
    private _default_host_list;
    private _ansible_meta_data;
    constructor(connection: Connection, context: WorkspaceFolderContext);
    initialize(): Promise<void>;
    set collections_paths(updatedCollectionPath: string[]);
    get collections_paths(): string[];
    set default_host_list(defaultHostList: string[]);
    get default_host_list(): string[];
    set module_locations(updatedModulesPath: string[]);
    get module_locations(): string[];
    get ansible_location(): string;
    get ansible_meta_data(): object;
}
