module.exports={A:{A:{"132":"J D E F A B xB"},B:{"1":"P Q R S T U V W Z a b c d e f g h i j k l H X m","132":"C K L G M N O"},C:{"1":"JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W Z a b c d e f g h i j k l H X m","33":"0 1 2 3 4 5 6 7 8 9 N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB","132":"yB mB I n J D E F zB 0B","292":"A B C K L G M"},D:{"1":"HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R S T U V W Z a b c d e f g h i j k l H X m 1B 2B 3B","132":"I n J D E F A B C K L G M","548":"0 1 2 3 4 5 6 7 8 9 N O o p q r s t u v w x y z AB BB CB DB EB FB GB"},E:{"132":"I n J D E 4B qB 5B 6B 7B","548":"F A B C K L G 8B rB jB kB 9B AC BC sB tB uB lB CC"},F:{"132":"0 1 2 3 4 5 6 7 8 9 F B C G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W DC EC FC GC jB vB HC kB"},G:{"132":"E qB IC wB JC KC LC MC","548":"NC OC PC QC RC SC TC UC VC WC XC YC ZC aC bC sB tB uB lB"},H:{"16":"cC"},I:{"1":"H","16":"mB I dC eC fC gC wB hC iC"},J:{"16":"D A"},K:{"1":"Y","16":"A B C jB vB kB"},L:{"1":"X"},M:{"1":"H"},N:{"132":"A B"},O:{"16":"jC"},P:{"1":"kC lC mC nC oC rB pC qC rC sC tC lB uC","16":"I"},Q:{"16":"vC"},R:{"16":"wC"},S:{"33":"xC"}},B:4,C:"CSS unicode-bidi property"};
