module.exports={A:{A:{"2":"J D E F A B xB"},B:{"1":"a b c d e f g h i j k l H X m","2":"C K L G M N O","1028":"W Z","1540":"P Q R S T U V"},C:{"1":"WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W Z a b c d e f g h i j k l H X m","2":"yB","164":"0 1 2 3 4 5 6 7 8 9 mB I n J D E F A B C K L G M N O o p q r s t u v w x y z zB 0B","1540":"AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB"},D:{"1":"a b c d e f g h i j k l H X m 1B 2B 3B","292":"0 1 2 3 4 5 6 7 8 9 I n J D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB","1028":"W Z","1540":"ZB aB bB cB dB eB fB gB hB iB P Q R S T U V"},E:{"1":"G BC sB tB uB lB CC","292":"I n J D E F A B C 4B qB 5B 6B 7B 8B rB jB","1028":"AC","1540":"K L kB 9B"},F:{"1":"gB hB iB P Q R pB S T U V W","2":"F B C DC EC FC GC jB vB HC kB","292":"0 1 2 3 4 5 6 7 8 9 G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB","1028":"eB fB","1540":"PB QB RB SB TB UB Y VB WB XB YB ZB aB bB cB dB"},G:{"1":"bC sB tB uB lB","292":"E qB IC wB JC KC LC MC NC OC PC QC RC SC TC","1028":"aC","1540":"UC VC WC XC YC ZC"},H:{"2":"cC"},I:{"1":"H","292":"mB I dC eC fC gC wB hC iC"},J:{"292":"D A"},K:{"1":"Y","2":"A B C jB vB kB"},L:{"1":"X"},M:{"1":"H"},N:{"2":"A B"},O:{"292":"jC"},P:{"1":"tC lB uC","292":"I kC lC mC nC oC","1540":"rB pC qC rC sC"},Q:{"1540":"vC"},R:{"1540":"wC"},S:{"1540":"xC"}},B:5,C:"CSS Logical Properties"};
