import { AllFormatOptions, CommonExportOptions, LinkProviders } from './opts';
import { Paragraph } from './dom';
export declare function addToDestination(destination: string[], paragraph: Paragraph, opts: AllFormatOptions & LinkProviders & CommonExportOptions): void;
//# sourceMappingURL=format.d.ts.map