module.exports={A:{A:{"2":"J D E xB","520":"F A B"},B:{"1":"P Q R S T U V W Z a b c d e f g h i j k l H X m","8":"C K","388":"L G M N O"},C:{"1":"0 1 2 3 4 5 6 7 8 9 x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W Z a b c d e f g h i j k l H X m","2":"yB mB zB 0B","132":"I n J D E F A B C K L G M N O o p q r s t u v w"},D:{"1":"0 1 2 3 4 5 6 7 8 9 u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R S T U V W Z a b c d e f g h i j k l H X m 1B 2B 3B","2":"I n","132":"J D E F A B C K L G M N O o p q r s t"},E:{"1":"lB CC","2":"4B","8":"I n qB 5B","520":"J D E F A B C 6B 7B 8B rB jB","1028":"K kB 9B","7172":"L","8196":"G AC BC sB tB uB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W","2":"F DC EC FC","132":"B C G GC jB vB HC kB"},G:{"2":"E qB IC wB JC KC LC MC NC OC PC QC RC SC TC","1028":"UC VC WC XC YC","3076":"ZC aC bC sB tB uB lB"},H:{"2":"cC"},I:{"1":"H","2":"dC eC","132":"mB I fC gC wB hC iC"},J:{"2":"D A"},K:{"1":"Y","2":"A B C jB vB kB"},L:{"1":"X"},M:{"1":"H"},N:{"8":"A B"},O:{"1":"jC"},P:{"1":"kC lC mC nC oC rB pC qC rC sC tC lB uC","132":"I"},Q:{"1":"vC"},R:{"1":"wC"},S:{"1":"xC"}},B:6,C:"WebM video format"};
