module.exports={A:{A:{"1":"F A B","132":"J D E xB"},B:{"1":"L G M N O P Q R S T U V W Z a b c d e f g h i j k l H X m","260":"C K"},C:{"1":"0 1 2 3 4 5 6 7 8 9 I n J D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W Z a b c d e f g h i j k l H X m","4":"yB mB zB 0B"},D:{"1":"0 1 2 3 4 5 6 7 8 9 n J D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R S T U V W Z a b c d e f g h i j k l H X m 1B 2B 3B","4":"I"},E:{"1":"n J D E F A B C K L G 5B 6B 7B 8B rB jB kB 9B AC BC sB tB uB lB CC","4":"I 4B qB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W","260":"F B C DC EC FC GC jB vB HC kB"},G:{"2":"E qB IC wB JC KC LC MC NC OC PC QC RC SC TC UC VC WC XC YC ZC aC bC sB tB uB lB"},H:{"2":"cC"},I:{"1":"H","2":"mB I dC eC fC gC wB hC iC"},J:{"2":"D","16":"A"},K:{"1":"Y","2":"A B C jB vB kB"},L:{"1":"X"},M:{"2":"H"},N:{"2":"A B"},O:{"2":"jC"},P:{"2":"I kC lC mC nC oC rB pC qC rC sC tC lB uC"},Q:{"2":"vC"},R:{"2":"wC"},S:{"2":"xC"}},B:4,C:"CSS3 Cursors (original values)"};
