module.exports={A:{A:{"2":"J D E F A B xB"},B:{"1":"P Q R S T U V W Z a b c d e f g h i j k l H X m","2":"C K L G M N O"},C:{"1":"IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W Z a b c d e f g h i j k l H X m","2":"0 1 2 3 4 5 6 7 8 9 yB mB I n J D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB zB 0B"},D:{"1":"TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R S T U V W Z a b c d e f g h i j k l H X m 1B 2B 3B","2":"0 1 2 3 4 5 6 7 8 9 I n J D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB","194":"LB MB NB OB PB QB RB nB SB oB"},E:{"1":"A B C K L G rB jB kB 9B AC BC sB tB uB lB CC","2":"I n J D E F 4B qB 5B 6B 7B 8B"},F:{"1":"LB MB NB OB PB QB RB SB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W","2":"0 1 2 3 4 5 6 7 F B C G M N O o p q r s t u v w x y z DC EC FC GC jB vB HC kB","194":"8 9 AB BB CB DB EB FB GB HB IB JB KB"},G:{"1":"PC QC RC SC TC UC VC WC XC YC ZC aC bC sB tB uB lB","2":"E qB IC wB JC KC LC MC NC OC"},H:{"2":"cC"},I:{"1":"H","2":"mB I dC eC fC gC wB hC iC"},J:{"2":"D A"},K:{"1":"Y","2":"A B C jB vB kB"},L:{"1":"X"},M:{"1":"H"},N:{"2":"A B"},O:{"2":"jC"},P:{"1":"nC oC rB pC qC rC sC tC lB uC","2":"I","194":"kC lC mC"},Q:{"2":"vC"},R:{"194":"wC"},S:{"2":"xC"}},B:7,C:"#rrggbbaa hex color notation"};
