module.exports={A:{A:{"388":"A B","900":"J D E F xB"},B:{"388":"C K L G M N O","900":"P Q R S T U V W Z a b c d e f g h i j k l H X m"},C:{"772":"VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W Z a b c d e f g h i j k l H X m","900":"0 1 2 3 4 5 6 7 8 9 yB mB I n J D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y zB 0B"},D:{"900":"0 1 2 3 4 5 6 7 8 9 I n J D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R S T U V W Z a b c d e f g h i j k l H X m 1B 2B 3B"},E:{"772":"A","900":"I n J D E F B C K L G 4B qB 5B 6B 7B 8B rB jB kB 9B AC BC sB tB uB lB CC"},F:{"16":"F DC","129":"B C EC FC GC jB vB HC kB","900":"0 1 2 3 4 5 6 7 8 9 G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W"},G:{"900":"E qB IC wB JC KC LC MC NC OC PC QC RC SC TC UC VC WC XC YC ZC aC bC sB tB uB lB"},H:{"129":"cC"},I:{"900":"mB I H dC eC fC gC wB hC iC"},J:{"900":"D A"},K:{"129":"A B C jB vB kB","900":"Y"},L:{"900":"X"},M:{"900":"H"},N:{"388":"A B"},O:{"900":"jC"},P:{"900":"I kC lC mC nC oC rB pC qC rC sC tC lB uC"},Q:{"900":"vC"},R:{"900":"wC"},S:{"900":"xC"}},B:2,C:"CSS page-break properties"};
