/**
Detect whether the terminal supports Unicode.

@example
```
import isUnicodeSupported = require('is-unicode-supported');

isUnicodeSupported();
//=> true
```
*/
declare function isUnicodeSupported(): boolean;

export = isUnicodeSupported;
