module.exports={A:{A:{"2":"xB","8":"J D E","129":"F A B"},B:{"1":"N O P Q R S T U V W Z a b c d e f g h i j k l H X m","129":"C K L G M"},C:{"1":"0 1 2 3 4 5 6 7 8 9 I n J D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W Z a b c d e f g h i j k l H X m","8":"yB mB zB 0B"},D:{"1":"0 1 2 3 4 5 6 7 8 9 D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R S T U V W Z a b c d e f g h i j k l H X m 1B 2B 3B","8":"I n J"},E:{"1":"F A B C K L G 8B rB jB kB 9B AC BC sB tB uB lB CC","8":"I n 4B qB","129":"J D E 5B 6B 7B"},F:{"1":"0 1 2 3 4 5 6 7 8 9 C G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W HC kB","2":"B GC jB vB","8":"F DC EC FC"},G:{"1":"NC OC PC QC RC SC TC UC VC WC XC YC ZC aC bC sB tB uB lB","8":"qB IC wB","129":"E JC KC LC MC"},H:{"1":"cC"},I:{"1":"H hC iC","2":"dC eC fC","129":"mB I gC wB"},J:{"1":"A","129":"D"},K:{"1":"C Y kB","8":"A B jB vB"},L:{"1":"X"},M:{"1":"H"},N:{"129":"A B"},O:{"1":"jC"},P:{"1":"I kC lC mC nC oC rB pC qC rC sC tC lB uC"},Q:{"1":"vC"},R:{"1":"wC"},S:{"1":"xC"}},B:1,C:"Inline SVG in HTML5"};
