module.exports={A:{A:{"2":"J D E xB","257":"F A B"},B:{"1":"C K L G M N O","513":"P Q R S T U V W Z a b c d e f g h i j k l H X m"},C:{"1":"1 2 3 4 5 6 7 8 9 AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W Z a b c d e f g h i j k l H X m","2":"0 yB mB I n J D E F A B C K L G M N O o p q r s t u v w x y z zB 0B"},D:{"2":"0 1 2 3 4 5 6 7 8 9 I n J D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB","513":"bB cB dB eB fB gB hB iB P Q R S T U V W Z a b c d e f g h i j k l H X m 1B 2B 3B"},E:{"1":"L G AC BC sB tB uB lB CC","2":"I n J D E F A 4B qB 5B 6B 7B 8B rB","129":"B C K jB kB 9B"},F:{"2":"0 1 2 3 4 5 6 7 8 9 F B C G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB DC EC FC GC jB vB HC kB","513":"RB SB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W"},G:{"1":"RC SC TC UC VC WC XC YC ZC aC bC sB tB uB lB","2":"E qB IC wB JC KC LC MC NC OC PC QC"},H:{"2":"cC"},I:{"1":"H","2":"mB I dC eC fC gC wB hC iC"},J:{"16":"D A"},K:{"1":"Y","2":"A B C jB vB kB"},L:{"1":"X"},M:{"1":"H"},N:{"16":"A B"},O:{"1":"jC"},P:{"1":"rB pC qC rC sC tC lB uC","2":"I kC lC mC nC oC"},Q:{"1":"vC"},R:{"1":"wC"},S:{"1":"xC"}},B:6,C:"COLR/CPAL(v0) Font Formats"};
