module.exports={A:{A:{"2":"J D E xB","260":"F A B"},B:{"132":"P Q R S T U V W Z a b c d e f g h i j k l H X m","260":"C K L G M N O"},C:{"2":"yB mB I n zB 0B","260":"0 1 2 3 4 5 6 7 8 9 J D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W Z a b c d e f g h i j k l H X m"},D:{"16":"I n J D E F A B C K L","132":"0 1 2 3 4 5 6 7 8 9 G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R S T U V W Z a b c d e f g h i j k l H X m 1B 2B 3B"},E:{"16":"4B qB","132":"I n J D E F A B C K L G 5B 6B 7B 8B rB jB kB 9B AC BC sB tB uB lB CC"},F:{"1":"C HC kB","2":"F DC EC FC GC","16":"B jB vB","132":"0 1 2 3 4 5 6 7 8 9 G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W"},G:{"16":"qB IC","132":"E wB JC KC LC MC NC OC PC QC RC SC TC UC VC WC XC YC ZC aC bC sB tB uB lB"},H:{"2":"cC"},I:{"16":"dC eC","132":"mB I H fC gC wB hC iC"},J:{"132":"D A"},K:{"1":"C kB","2":"A","16":"B jB vB","132":"Y"},L:{"132":"X"},M:{"260":"H"},N:{"260":"A B"},O:{"132":"jC"},P:{"132":"I kC lC mC nC oC rB pC qC rC sC tC lB uC"},Q:{"132":"vC"},R:{"132":"wC"},S:{"260":"xC"}},B:5,C:"Mutation events"};
