/**
Lowercase the keys of an object.

@returns A new object with the keys lowercased.

@example
```
import lowercaseKeys from 'lowercase-keys';

lowercaseKeys({FOO: true, bAr: false});
//=> {foo: true, bar: false}
```
*/
export default function lowercaseKeys<T>(object: Record<string, T>): Record<string, T>;
