module.exports={A:{A:{"2":"J D E F A B xB"},B:{"2":"C K L G M N","260":"P Q R S T U V W Z a b c d e f g h i j k l H X m","3138":"O"},C:{"1":"NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W Z a b c d e f g h i j k l H X m","2":"yB mB","132":"0 1 2 3 4 5 6 7 8 9 I n J D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB zB 0B","644":"GB HB IB JB KB LB MB"},D:{"2":"I n J D E F A B C K L G M N O o p q r s","260":"OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R S T U V W Z a b c d e f g h i j k l H X m 1B 2B 3B","292":"0 1 2 3 4 5 6 7 8 9 t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB"},E:{"2":"I n J 4B qB 5B 6B","260":"L G 9B AC BC sB tB uB lB CC","292":"D E F A B C K 7B 8B rB jB kB"},F:{"2":"F B C DC EC FC GC jB vB HC kB","260":"BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W","292":"0 1 2 3 4 5 6 7 8 9 G M N O o p q r s t u v w x y z AB"},G:{"2":"qB IC wB JC KC","260":"VC WC XC YC ZC aC bC sB tB uB lB","292":"E LC MC NC OC PC QC RC SC TC UC"},H:{"2":"cC"},I:{"2":"mB I dC eC fC gC wB","260":"H","292":"hC iC"},J:{"2":"D A"},K:{"2":"A B C jB vB kB","260":"Y"},L:{"260":"X"},M:{"1":"H"},N:{"2":"A B"},O:{"292":"jC"},P:{"292":"I kC lC mC nC oC rB pC qC rC sC tC lB uC"},Q:{"292":"vC"},R:{"260":"wC"},S:{"644":"xC"}},B:4,C:"CSS clip-path property (for HTML)"};
