module.exports={A:{A:{"2":"J D E F A B xB"},B:{"1":"P Q R S T U V W Z a b c d e f g h i j k l H X m","2":"C K L G M N O"},C:{"1":"c d e f g h i j k l H X m","2":"yB mB zB 0B","33":"MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W Z a b","164":"0 1 2 3 4 5 6 7 8 9 I n J D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB"},D:{"1":"BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R S T U V W Z a b c d e f g h i j k l H X m 1B 2B 3B","2":"I n J D E F A B C K L G M N O o p","132":"0 1 2 3 4 5 6 7 8 9 q r s t u v w x y z AB"},E:{"1":"L G 9B AC BC sB tB uB lB CC","2":"I n J 4B qB 5B","132":"D E F A B C K 6B 7B 8B rB jB kB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W","2":"F DC EC FC","132":"G M N O o p q r s t u v w x","164":"B C GC jB vB HC kB"},G:{"1":"YC ZC aC bC sB tB uB lB","2":"qB IC wB JC KC","132":"E LC MC NC OC PC QC RC SC TC UC VC WC XC"},H:{"164":"cC"},I:{"1":"H","2":"mB I dC eC fC gC wB","132":"hC iC"},J:{"132":"D A"},K:{"1":"Y","2":"A","164":"B C jB vB kB"},L:{"1":"X"},M:{"1":"H"},N:{"2":"A B"},O:{"1":"jC"},P:{"1":"I kC lC mC nC oC rB pC qC rC sC tC lB uC"},Q:{"1":"vC"},R:{"1":"wC"},S:{"164":"xC"}},B:4,C:"CSS3 tab-size"};
