import * as ASTUtils from './ast-utils';
import * as ESLintUtils from './eslint-utils';
import * as JSONSchema from './json-schema';
import * as TSESLint from './ts-eslint';
import * as TSESLintScope from './ts-eslint-scope';
export { ASTUtils, ESLintUtils, JSONSchema, TSESLint, TSESLintScope };
export * from './ts-estree';
//# sourceMappingURL=index.d.ts.map
