module.exports={A:{A:{"644":"J D E F xB","772":"A B"},B:{"1":"O P Q R S T U V W Z a b c d e f g h i j k l H X m","260":"C K L G M N"},C:{"1":"0 1 2 3 4 5 6 7 8 9 I n J D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W Z a b c d e f g h i j k l H X m zB 0B","8":"yB mB"},D:{"1":"0 1 2 3 4 5 6 7 8 9 I n J D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R S T U V W Z a b c d e f g h i j k l H X m 1B 2B 3B"},E:{"1":"I n J D E F A B C K L G 4B qB 5B 6B 7B 8B rB jB kB 9B AC BC sB tB uB lB CC"},F:{"1":"0 1 2 3 4 5 6 7 8 9 C G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W kB","8":"F B DC EC FC GC jB vB HC"},G:{"1":"bC sB tB uB lB","2":"E qB IC wB JC KC LC MC NC OC PC QC RC SC TC UC VC WC XC YC ZC aC"},H:{"2":"cC"},I:{"2":"mB I dC eC fC gC wB hC iC","1025":"H"},J:{"2":"D A"},K:{"1":"kB","8":"A B C jB vB","1025":"Y"},L:{"1025":"X"},M:{"2":"H"},N:{"1":"A B"},O:{"2":"jC"},P:{"2":"I kC lC mC nC oC rB pC qC rC sC tC lB uC"},Q:{"1":"vC"},R:{"2":"wC"},S:{"2":"xC"}},B:1,C:"Drag and Drop"};
