import { AnsibleDocTextOptions } from './opts';
import { Paragraph } from './dom';
export declare function toAnsibleDocText(paragraphs: Paragraph[], opts?: AnsibleDocTextOptions): string;
//# sourceMappingURL=ansible-doc-text.d.ts.map