export declare function parseEscapedArgs(input: string, index: number, count: number): [string[], number, string | undefined];
export declare function parseUnescapedArgs(input: string, index: number, count: number): [string[], number, string | undefined];
//# sourceMappingURL=parser-impl.d.ts.map