module.exports={A:{A:{"2":"J D E F A B xB"},B:{"2":"C K L G M N O","1537":"P Q R S T U V W Z a b c d e f g h i j k l H X m"},C:{"2":"yB","932":"0 1 2 3 4 5 6 7 8 9 mB I n J D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB zB 0B","2308":"WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W Z a b c d e f g h i j k l H X m"},D:{"2":"I n J D E F A B C K L G M N O o p q","545":"0 1 2 3 4 5 6 7 8 9 r s t u v w x y z AB BB CB DB EB","1537":"FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R S T U V W Z a b c d e f g h i j k l H X m 1B 2B 3B"},E:{"2":"I n J 4B qB 5B","516":"B C K L G jB kB 9B AC BC sB tB uB lB CC","548":"F A 8B rB","676":"D E 6B 7B"},F:{"2":"F B C DC EC FC GC jB vB HC kB","513":"3","545":"0 1 G M N O o p q r s t u v w x y z","1537":"2 4 5 6 7 8 9 AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W"},G:{"2":"qB IC wB JC KC","516":"ZC aC bC sB tB uB lB","548":"NC OC PC QC RC SC TC UC VC WC XC YC","676":"E LC MC"},H:{"2":"cC"},I:{"2":"mB I dC eC fC gC wB","545":"hC iC","1537":"H"},J:{"2":"D","545":"A"},K:{"2":"A B C jB vB kB","1537":"Y"},L:{"1537":"X"},M:{"2308":"H"},N:{"2":"A B"},O:{"1":"jC"},P:{"545":"I","1537":"kC lC mC nC oC rB pC qC rC sC tC lB uC"},Q:{"545":"vC"},R:{"1537":"wC"},S:{"932":"xC"}},B:5,C:"Intrinsic & Extrinsic Sizing"};
