export declare function toDotNotation(obj: any, res?: {}, current?: string): {};
export declare function structureSettings(settingsInDotNotation: any): any;
export declare function makeSettingsUnique(arrayObject: any): any;
