declare const _default: ((path: string | string[], opt?: import("./index.js").RimrafAsyncOptions | undefined) => Promise<boolean>) & {
    rimraf: ((path: string | string[], opt?: import("./index.js").RimrafAsyncOptions | undefined) => Promise<boolean>) & {
        sync: (path: string | string[], opt?: import("./index.js").RimrafSyncOptions | undefined) => boolean;
    };
    sync: (path: string | string[], opt?: import("./index.js").RimrafSyncOptions | undefined) => boolean;
    rimrafSync: (path: string | string[], opt?: import("./index.js").RimrafSyncOptions | undefined) => boolean;
    manual: ((path: string | string[], opt?: import("./index.js").RimrafAsyncOptions | undefined) => Promise<boolean>) & {
        sync: (path: string | string[], opt?: import("./index.js").RimrafSyncOptions | undefined) => boolean;
    };
    manualSync: (path: string | string[], opt?: import("./index.js").RimrafSyncOptions | undefined) => boolean;
    native: ((path: string | string[], opt?: import("./index.js").RimrafAsyncOptions | undefined) => Promise<boolean>) & {
        sync: (path: string | string[], opt?: import("./index.js").RimrafSyncOptions | undefined) => boolean;
    };
    nativeSync: (path: string | string[], opt?: import("./index.js").RimrafSyncOptions | undefined) => boolean;
    posix: ((path: string | string[], opt?: import("./index.js").RimrafAsyncOptions | undefined) => Promise<boolean>) & {
        sync: (path: string | string[], opt?: import("./index.js").RimrafSyncOptions | undefined) => boolean;
    };
    posixSync: (path: string | string[], opt?: import("./index.js").RimrafSyncOptions | undefined) => boolean;
    windows: ((path: string | string[], opt?: import("./index.js").RimrafAsyncOptions | undefined) => Promise<boolean>) & {
        sync: (path: string | string[], opt?: import("./index.js").RimrafSyncOptions | undefined) => boolean;
    };
    windowsSync: (path: string | string[], opt?: import("./index.js").RimrafSyncOptions | undefined) => boolean;
    moveRemove: ((path: string | string[], opt?: import("./index.js").RimrafAsyncOptions | undefined) => Promise<boolean>) & {
        sync: (path: string | string[], opt?: import("./index.js").RimrafSyncOptions | undefined) => boolean;
    };
    moveRemoveSync: (path: string | string[], opt?: import("./index.js").RimrafSyncOptions | undefined) => boolean;
} & {
    default: ((path: string | string[], opt?: import("./index.js").RimrafAsyncOptions | undefined) => Promise<boolean>) & {
        rimraf: ((path: string | string[], opt?: import("./index.js").RimrafAsyncOptions | undefined) => Promise<boolean>) & {
            sync: (path: string | string[], opt?: import("./index.js").RimrafSyncOptions | undefined) => boolean;
        };
        sync: (path: string | string[], opt?: import("./index.js").RimrafSyncOptions | undefined) => boolean;
        rimrafSync: (path: string | string[], opt?: import("./index.js").RimrafSyncOptions | undefined) => boolean;
        manual: ((path: string | string[], opt?: import("./index.js").RimrafAsyncOptions | undefined) => Promise<boolean>) & {
            sync: (path: string | string[], opt?: import("./index.js").RimrafSyncOptions | undefined) => boolean;
        };
        manualSync: (path: string | string[], opt?: import("./index.js").RimrafSyncOptions | undefined) => boolean;
        native: ((path: string | string[], opt?: import("./index.js").RimrafAsyncOptions | undefined) => Promise<boolean>) & {
            sync: (path: string | string[], opt?: import("./index.js").RimrafSyncOptions | undefined) => boolean;
        };
        nativeSync: (path: string | string[], opt?: import("./index.js").RimrafSyncOptions | undefined) => boolean;
        posix: ((path: string | string[], opt?: import("./index.js").RimrafAsyncOptions | undefined) => Promise<boolean>) & {
            sync: (path: string | string[], opt?: import("./index.js").RimrafSyncOptions | undefined) => boolean;
        };
        posixSync: (path: string | string[], opt?: import("./index.js").RimrafSyncOptions | undefined) => boolean;
        windows: ((path: string | string[], opt?: import("./index.js").RimrafAsyncOptions | undefined) => Promise<boolean>) & {
            sync: (path: string | string[], opt?: import("./index.js").RimrafSyncOptions | undefined) => boolean;
        };
        windowsSync: (path: string | string[], opt?: import("./index.js").RimrafSyncOptions | undefined) => boolean;
        moveRemove: ((path: string | string[], opt?: import("./index.js").RimrafAsyncOptions | undefined) => Promise<boolean>) & {
            sync: (path: string | string[], opt?: import("./index.js").RimrafSyncOptions | undefined) => boolean;
        };
        moveRemoveSync: (path: string | string[], opt?: import("./index.js").RimrafSyncOptions | undefined) => boolean;
    };
};
export = _default;
//# sourceMappingURL=index-cjs.d.ts.map