module.exports={A:{A:{"2":"J D E F A B xB"},B:{"2":"C K L G M N O","16":"P Q R S T U V W Z a b c d e f g h i j k l H X m"},C:{"2":"0 1 2 3 4 5 6 7 8 9 yB mB I n J D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W Z a b c d e f g h i j k l H zB 0B","16":"X m"},D:{"2":"0 1 2 3 4 5 6 7 8 9 I n J D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R S T U V W Z a b c d e f g h i j k l H X m","16":"1B 2B 3B"},E:{"1":"C K kB","2":"I n J D E F A B 4B qB 5B 6B 7B 8B rB jB","16":"L G 9B AC BC sB tB uB lB CC"},F:{"2":"0 1 2 3 4 5 6 7 8 9 F B C G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB DC EC FC GC jB vB HC kB","16":"MB NB OB PB QB RB SB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W"},G:{"2":"E qB IC wB JC KC LC MC NC OC PC QC RC SC TC UC VC WC XC YC ZC aC bC sB tB uB lB"},H:{"16":"cC"},I:{"2":"mB I dC eC fC gC wB hC iC","16":"H"},J:{"2":"D","16":"A"},K:{"2":"A B C jB vB kB","16":"Y"},L:{"16":"X"},M:{"16":"H"},N:{"2":"A","16":"B"},O:{"16":"jC"},P:{"2":"I kC lC","16":"mC nC oC rB pC qC rC sC tC lB uC"},Q:{"16":"vC"},R:{"16":"wC"},S:{"2":"xC"}},B:1,C:"Password Rules"};
