module.exports={A:{A:{"2":"F A B xB","8":"J D E"},B:{"1":"P Q R S T U V W Z a b c d e f g h i j k l H X m","2":"C K L G M N O"},C:{"1":"IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W Z a b c d e f g h i j k l H X m","2":"yB","8":"0 1 2 3 4 5 6 7 8 9 mB I n J D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB zB 0B","194":"GB HB"},D:{"1":"5 6 7 8 9 AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R S T U V W Z a b c d e f g h i j k l H X m 1B 2B 3B","8":"I n J D E F A B","257":"0 1 2 3 4 o p q r s t u v w x y z","769":"C K L G M N O"},E:{"1":"C K L G kB 9B AC BC sB tB uB lB CC","8":"I n 4B qB 5B","257":"J D E F A 6B 7B 8B","1025":"B rB jB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W","2":"C jB vB HC kB","8":"F B DC EC FC GC"},G:{"1":"E KC LC MC NC OC SC TC UC VC WC XC YC ZC aC bC sB tB uB lB","8":"qB IC wB JC","1025":"PC QC RC"},H:{"8":"cC"},I:{"1":"I H gC wB hC iC","8":"mB dC eC fC"},J:{"1":"A","8":"D"},K:{"1":"Y","8":"A B C jB vB kB"},L:{"1":"X"},M:{"1":"H"},N:{"2":"A B"},O:{"769":"jC"},P:{"1":"I kC lC mC nC oC rB pC qC rC sC tC lB uC"},Q:{"1":"vC"},R:{"1":"wC"},S:{"1":"xC"}},B:1,C:"Details & Summary elements"};
