"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAnsibleCommandExecPath = void 0;
// utils function to resolve executable path
const path = __importStar(require("path"));
/**
 * A method to return the path to the provided executable
 * @param name String representing the name of the ansible executable
 * @param settings The settings received from client
 * @returns Complete path of the ansible executable (string)
 */
function getAnsibleCommandExecPath(name, settings) {
    return name === "ansible-lint"
        ? settings.validation.lint.path
        : path.join(path.dirname(settings.ansible.path), name);
}
exports.getAnsibleCommandExecPath = getAnsibleCommandExecPath;
//# sourceMappingURL=execPath.js.map