module.exports={A:{A:{"2":"xB","8":"J D E F","260":"A B"},B:{"1":"P Q R S T U V W Z a b c d e f g h i j k l H X m","260":"C K L G","1284":"M N O"},C:{"8":"yB mB zB 0B","4612":"0 1 2 3 4 5 6 7 8 9 I n J D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W Z a b c d e f g h i j k l H X m"},D:{"1":"ZB aB bB cB dB eB fB gB hB iB P Q R S T U V W Z a b c d e f g h i j k l H X m 1B 2B 3B","8":"I n J D E F A B C K L G M N O o","132":"0 1 2 3 4 5 6 7 8 9 p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB"},E:{"1":"K L G kB 9B AC BC sB tB uB lB CC","8":"I n J D E F A B C 4B qB 5B 6B 7B 8B rB jB"},F:{"1":"F B C Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W DC EC FC GC jB vB HC kB","132":"0 1 2 3 4 5 6 7 8 9 G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB"},G:{"8":"E qB IC wB JC KC LC MC NC OC PC QC RC SC TC","2049":"UC VC WC XC YC ZC aC bC sB tB uB lB"},H:{"2":"cC"},I:{"1":"H iC","8":"mB I dC eC fC gC wB hC"},J:{"1":"A","8":"D"},K:{"1":"A B C Y jB vB kB"},L:{"1":"X"},M:{"516":"H"},N:{"8":"A B"},O:{"8":"jC"},P:{"1":"I kC lC mC nC oC rB pC qC rC sC tC lB uC"},Q:{"132":"vC"},R:{"1":"wC"},S:{"2":"xC"}},B:1,C:"Datalist element"};
