module.exports={A:{A:{"2":"J D E F A B xB"},B:{"1":"T U V W Z a b c d e f g h i j k l H X m","2":"C K L G M N O","260":"P Q R S"},C:{"1":"R pB S T U V W Z a b c d e f g h i j k l H X m","2":"0 1 yB mB I n J D E F A B C K L G M N O o p q r s t u v w x y z zB 0B","260":"nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB","516":"GB HB IB JB KB LB MB NB OB PB QB RB","580":"2 3 4 5 6 7 8 9 AB BB CB DB EB FB","2049":"fB gB hB iB P Q"},D:{"1":"T U V W Z a b c d e f g h i j k l H X m 1B 2B 3B","2":"0 1 2 3 4 I n J D E F A B C K L G M N O o p q r s t u v w x y z","132":"5 6 7","260":"8 9 AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R S"},E:{"1":"G BC sB tB uB lB CC","2":"I n J D E F A 4B qB 5B 6B 7B 8B rB","1090":"B C K jB kB","2049":"L 9B AC"},F:{"1":"bB cB dB eB fB gB hB iB P Q R pB S T U V W","2":"F B C G M N O o p q r DC EC FC GC jB vB HC kB","132":"s t u","260":"0 1 2 3 4 5 6 7 8 9 v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB Y VB WB XB YB ZB aB"},G:{"2":"E qB IC wB JC KC LC MC NC OC PC QC","1090":"RC SC TC UC VC WC XC","2049":"YC ZC aC bC sB tB uB lB"},H:{"2":"cC"},I:{"1":"H","2":"mB I dC eC fC gC wB hC iC"},J:{"2":"D A"},K:{"1":"Y","2":"A B C jB vB kB"},L:{"1":"X"},M:{"1":"H"},N:{"2":"A B"},O:{"260":"jC"},P:{"260":"I kC lC mC nC oC rB pC qC rC sC tC lB uC"},Q:{"260":"vC"},R:{"260":"wC"},S:{"516":"xC"}},B:5,C:"Web Animations API"};
