module.exports={A:{A:{"1":"F A B","2":"J D E xB"},B:{"1":"C K L G M N O P Q R S T U V W Z a b c d e f g h i j k l H X m"},C:{"1":"TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W Z a b c d e f g h i j k l H X m","33":"0 1 2 3 4 5 6 7 8 9 yB mB I n J D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB zB 0B"},D:{"1":"0 1 2 3 4 5 6 7 8 9 I n J D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R S T U V W Z a b c d e f g h i j k l H X m 1B 2B 3B"},E:{"1":"I n J D E F A B C K L G 4B qB 5B 6B 7B 8B rB jB kB 9B AC BC sB tB uB lB CC"},F:{"1":"0 1 2 3 4 5 6 7 8 9 B C G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W DC EC FC GC jB vB HC kB","2":"F"},G:{"2":"E qB IC wB JC KC LC MC NC OC PC QC RC SC TC UC VC WC XC YC ZC aC bC sB tB uB lB"},H:{"2":"cC"},I:{"1":"H hC iC","2":"mB I dC eC fC gC wB"},J:{"1":"A","2":"D"},K:{"1":"C Y vB kB","16":"A B jB"},L:{"1":"X"},M:{"1":"H"},N:{"1":"A B"},O:{"1":"jC"},P:{"1":"I kC lC mC nC oC rB pC qC rC sC tC lB uC"},Q:{"1":"vC"},R:{"1":"wC"},S:{"33":"xC"}},B:5,C:"::selection CSS pseudo-element"};
