import SemVer = require('../classes/semver');
import semver = require('../index');

/**
 * Return the patch version number.
 */
declare function patch(version: string | SemVer, optionsOrLoose?: boolean | semver.Options): number;

export = patch;
