module.exports={A:{A:{"1":"B","4":"J D E F A xB"},B:{"1":"C K L G M","129":"N O P Q R S T U V W Z a b c d e f g h i j k l H X m"},C:{"1":"0 1 2 3 4 5 6 7 8 9 J D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB","4":"yB mB I n zB 0B","129":"KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W Z a b c d e f g h i j k l H X m"},D:{"1":"EB FB GB HB IB JB KB LB MB NB","4":"I n J","129":"0 1 2 3 4 5 6 7 8 9 D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R S T U V W Z a b c d e f g h i j k l H X m 1B 2B 3B"},E:{"4":"I n 4B qB","129":"J D E F A B C K L G 5B 6B 7B 8B rB jB kB 9B AC BC sB tB uB lB CC"},F:{"1":"1 2 3 4 5 6 7 8 9 C AB jB vB HC kB","4":"F B DC EC FC GC","129":"0 G M N O o p q r s t u v w x y z BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W"},G:{"4":"qB IC wB","129":"E JC KC LC MC NC OC PC QC RC SC TC UC VC WC XC YC ZC aC bC sB tB uB lB"},H:{"4":"cC"},I:{"4":"dC eC fC","129":"mB I H gC wB hC iC"},J:{"129":"D A"},K:{"1":"C jB vB kB","4":"A B","129":"Y"},L:{"129":"X"},M:{"129":"H"},N:{"1":"B","4":"A"},O:{"129":"jC"},P:{"129":"I kC lC mC nC oC rB pC qC rC sC tC lB uC"},Q:{"1":"vC"},R:{"129":"wC"},S:{"1":"xC"}},B:1,C:"dataset & data-* attributes"};
