import { HTMLOptions } from './opts';
import { Paragraph } from './dom';
export declare function quoteHTML(text: string): string;
export declare function quoteHTMLArg(text: string): string;
export declare function toHTML(paragraphs: Paragraph[], opts?: HTMLOptions): string;
//# sourceMappingURL=html.d.ts.map