import { MDOptions } from './opts';
import { Paragraph } from './dom';
export declare function quoteMD(text: string): string;
export declare function toMD(paragraphs: Paragraph[], opts?: MDOptions): string;
//# sourceMappingURL=md.d.ts.map