module.exports={A:{A:{"2":"J D E F A B xB"},B:{"2":"C K L","33":"P Q R S T U V W Z a b c d e f g h i j k l H X m","161":"G M N O"},C:{"2":"0 1 2 3 4 5 6 7 8 9 yB mB I n J D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB zB 0B","161":"IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W Z a b c d e f g h i j k l H X m","450":"HB"},D:{"33":"0 1 2 3 4 5 6 7 8 9 I n J D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R S T U V W Z a b c d e f g h i j k l H X m 1B 2B 3B"},E:{"33":"I n J D E F A B C K L G 4B qB 5B 6B 7B 8B rB jB kB 9B AC BC sB tB uB lB CC"},F:{"2":"F B C DC EC FC GC jB vB HC kB","33":"0 1 2 3 4 5 6 7 8 9 G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W"},G:{"33":"E IC wB JC KC LC MC NC OC PC QC RC SC TC UC VC WC XC YC ZC aC bC sB tB uB lB","36":"qB"},H:{"2":"cC"},I:{"2":"mB","33":"I H dC eC fC gC wB hC iC"},J:{"33":"D A"},K:{"2":"A B C jB vB kB","33":"Y"},L:{"33":"X"},M:{"161":"H"},N:{"2":"A B"},O:{"33":"jC"},P:{"33":"I kC lC mC nC oC rB pC qC rC sC tC lB uC"},Q:{"33":"vC"},R:{"33":"wC"},S:{"161":"xC"}},B:7,C:"CSS text-stroke and text-fill"};
