import { PackageFile } from '../types/PackageFile';
/** Gets a list of dependency sections based on options.dep. */
declare const resolveDepSections: (dep?: string | string[]) => (keyof PackageFile)[];
export default resolveDepSections;
