module.exports={A:{A:{"1":"J D E F A xB","132":"B"},B:{"132":"C K L G M N O","260":"P Q R S T U V W Z a b c d e f g h i j k l H X m"},C:{"1":"yB mB I n J D E F A B C K L G M N O o p q r s t u v w x y zB 0B","516":"0 1 2 3 4 5 6 7 8 9 z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W Z a b c d e f g h i j k l H X m"},D:{"1":"N O o p q r s t u v","2":"I n J D E F A B C K L G M","132":"0 1 2 3 4 5 6 7 8 9 w x y z","260":"AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R S T U V W Z a b c d e f g h i j k l H X m 1B 2B 3B"},E:{"1":"J 5B 6B","2":"I n 4B qB","2052":"D E F A B C K L G 7B 8B rB jB kB 9B AC BC sB tB uB lB CC"},F:{"1":"0 1 2 3 4 5 6 7 8 9 F B C G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W DC EC FC GC jB vB HC kB"},G:{"2":"qB IC wB","1025":"E JC KC LC MC NC OC PC QC RC SC TC UC VC WC XC YC ZC aC bC sB tB uB lB"},H:{"1025":"cC"},I:{"1":"mB I H dC eC fC gC wB hC iC"},J:{"1":"D A"},K:{"1":"A B C Y jB vB kB"},L:{"1":"X"},M:{"1":"H"},N:{"2052":"A B"},O:{"1025":"jC"},P:{"1":"I kC lC mC nC oC rB pC qC rC sC tC lB uC"},Q:{"260":"vC"},R:{"1":"wC"},S:{"516":"xC"}},B:1,C:"autocomplete attribute: on & off values"};
