module.exports={A:{A:{"2":"xB","8":"J D E F A","129":"B"},B:{"1":"P Q R S T U V W Z a b c d e f g h i j k l H X m","129":"C K L G M N O"},C:{"1":"0 1 2 3 4 5 6 7 8 9 t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W Z a b c d e f g h i j k l H X m","2":"yB mB zB 0B","129":"I n J D E F A B C K L G M N O o p q r s"},D:{"1":"2 3 4 5 6 7 8 9 AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R S T U V W Z a b c d e f g h i j k l H X m 1B 2B 3B","2":"I n J D","129":"0 1 E F A B C K L G M N O o p q r s t u v w x y z"},E:{"1":"E F A B C K L G 8B rB jB kB 9B AC BC sB tB uB lB CC","2":"I n 4B qB","129":"J D 5B 6B 7B"},F:{"1":"0 1 2 3 4 5 6 7 8 9 o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W","2":"F B DC EC FC GC jB vB HC","129":"C G M N O kB"},G:{"1":"E MC NC OC PC QC RC SC TC UC VC WC XC YC ZC aC bC sB tB uB lB","2":"qB IC wB JC KC LC"},H:{"2":"cC"},I:{"1":"H","2":"mB I dC eC fC gC wB hC iC"},J:{"1":"A","2":"D"},K:{"1":"C Y kB","2":"A B jB vB"},L:{"1":"X"},M:{"1":"H"},N:{"8":"A","129":"B"},O:{"129":"jC"},P:{"1":"I kC lC mC nC oC rB pC qC rC sC tC lB uC"},Q:{"1":"vC"},R:{"1":"wC"},S:{"129":"xC"}},B:6,C:"WebGL - 3D Canvas graphics"};
