import { TSESTreeOptions } from '../parser-options';
import { MutableParseSettings } from './index';
export declare function createParseSettings(code: string, options?: Partial<TSESTreeOptions>): MutableParseSettings;
export declare function clearTSConfigMatchCache(): void;
//# sourceMappingURL=createParseSettings.d.ts.map
