module.exports={A:{A:{"1":"F A B","16":"xB","260":"J D E"},B:{"1":"C K L G M N O P Q R S T U V W Z a b c d e f g h i j k l H X m"},C:{"1":"LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W Z a b c d e f g h i j k l H X m","132":"0 1 2 3 4 5 6 7 8 9 yB mB I n J D E F A B C K L G M N O o p q r s t u v w x y z AB BB zB 0B","2180":"CB DB EB FB GB HB IB JB KB"},D:{"1":"0 1 2 3 4 5 6 7 8 9 G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R S T U V W Z a b c d e f g h i j k l H X m 1B 2B 3B","16":"I n J D E F A B C K L"},E:{"1":"J D E F A B C K L G 5B 6B 7B 8B rB jB kB 9B AC BC sB tB uB lB CC","16":"I n 4B qB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W","132":"F B C DC EC FC GC jB vB HC kB"},G:{"16":"wB","132":"qB IC","516":"E JC KC LC MC NC OC PC QC RC SC TC UC VC WC XC YC ZC aC bC sB tB uB lB"},H:{"2":"cC"},I:{"1":"H hC iC","16":"mB I dC eC fC gC","1025":"wB"},J:{"1":"A","16":"D"},K:{"1":"Y","16":"A B C jB vB","132":"kB"},L:{"1":"X"},M:{"1":"H"},N:{"1":"B","16":"A"},O:{"1025":"jC"},P:{"1":"I kC lC mC nC oC rB pC qC rC sC tC lB uC"},Q:{"1":"vC"},R:{"1":"wC"},S:{"2180":"xC"}},B:5,C:"Selection API"};
