module.exports={A:{A:{"2":"J D E F xB","129":"A B"},B:{"1":"P Q R S T U V W Z a b c d e f g h i j k l H X m","129":"C K","1025":"L G M N O"},C:{"2":"yB mB I n J D E F A B C K L G M N O o p q r s t u v w x zB 0B","513":"0 1 2 3 4 5 6 7 8 9 y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W Z a b c d e f g h i j k l H X m"},D:{"1":"0 1 2 3 4 5 6 7 8 9 J D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R S T U V W Z a b c d e f g h i j k l H X m 1B 2B 3B","2":"I n"},E:{"1":"n J D E F A B C K L G 5B 6B 7B 8B rB jB kB 9B AC BC sB tB uB lB CC","2":"I 4B qB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 F B C G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W DC EC FC GC jB vB HC kB"},G:{"388":"E qB IC wB JC KC LC MC NC OC PC QC RC SC TC UC VC WC XC YC ZC aC bC sB tB uB lB"},H:{"2":"cC"},I:{"2":"mB dC eC fC","388":"I H gC wB hC iC"},J:{"2":"D","388":"A"},K:{"1":"A B C jB vB kB","388":"Y"},L:{"388":"X"},M:{"641":"H"},N:{"388":"A B"},O:{"388":"jC"},P:{"388":"I kC lC mC nC oC rB pC qC rC sC tC lB uC"},Q:{"388":"vC"},R:{"388":"wC"},S:{"513":"xC"}},B:1,C:"Number input type"};
