import { Index } from '../types/IndexType';
import { Options } from '../types/Options';
/** Fetches the list of all installed packages. */
export declare const list: (options?: Options) => Promise<Index<string | undefined>>;
export { defaultPrefix, distTag, getPeerDependencies, greatest, latest, minor, patch, newest, packageAuthorChanged, } from './npm';
