module.exports={A:{A:{"2":"xB","8":"J D E","129":"A B","161":"F"},B:{"1":"N O P Q R S T U V W Z a b c d e f g h i j k l H X m","129":"C K L G M"},C:{"1":"0 1 2 3 4 5 6 7 8 9 M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W Z a b c d e f g h i j k l H X m","2":"yB mB","33":"I n J D E F A B C K L G zB 0B"},D:{"1":"5 6 7 8 9 AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R S T U V W Z a b c d e f g h i j k l H X m 1B 2B 3B","33":"0 1 2 3 4 I n J D E F A B C K L G M N O o p q r s t u v w x y z"},E:{"1":"F A B C K L G 8B rB jB kB 9B AC BC sB tB uB lB CC","33":"I n J D E 4B qB 5B 6B 7B"},F:{"1":"0 1 2 3 4 5 6 7 8 9 s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W kB","2":"F DC EC","33":"B C G M N O o p q r FC GC jB vB HC"},G:{"1":"NC OC PC QC RC SC TC UC VC WC XC YC ZC aC bC sB tB uB lB","33":"E qB IC wB JC KC LC MC"},H:{"2":"cC"},I:{"1":"H","33":"mB I dC eC fC gC wB hC iC"},J:{"33":"D A"},K:{"1":"B C Y jB vB kB","2":"A"},L:{"1":"X"},M:{"1":"H"},N:{"1":"A B"},O:{"1":"jC"},P:{"1":"I kC lC mC nC oC rB pC qC rC sC tC lB uC"},Q:{"1":"vC"},R:{"1":"wC"},S:{"1":"xC"}},B:4,C:"CSS3 2D Transforms"};
