module.exports={A:{A:{"1":"A B","2":"J D E F xB"},B:{"1":"C K L G M N O","516":"P Q R S T U V W Z a b c d e f g h i j k l H X m"},C:{"132":"LB MB NB OB PB QB RB nB SB oB TB UB Y","164":"0 1 2 3 4 5 6 7 8 9 yB mB I n J D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB zB 0B","516":"VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W Z a b c","1028":"d e f g h i j k l H X m"},D:{"420":"0 1 2 3 4 5 6 7 8 9 I n J D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB","516":"JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R S T U V W Z a b c d e f g h i j k l H X m 1B 2B 3B"},E:{"1":"A B C K L G rB jB kB 9B AC BC sB tB uB lB CC","132":"F 8B","164":"D E 7B","420":"I n J 4B qB 5B 6B"},F:{"1":"C jB vB HC kB","2":"F B DC EC FC GC","420":"0 1 2 3 4 5 G M N O o p q r s t u v w x y z","516":"6 7 8 9 AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W"},G:{"1":"PC QC RC SC TC UC VC WC XC YC ZC aC bC sB tB uB lB","132":"NC OC","164":"E LC MC","420":"qB IC wB JC KC"},H:{"1":"cC"},I:{"420":"mB I dC eC fC gC wB hC iC","516":"H"},J:{"420":"D A"},K:{"1":"C jB vB kB","2":"A B","516":"Y"},L:{"516":"X"},M:{"516":"H"},N:{"1":"A B"},O:{"1":"jC"},P:{"1":"kC lC mC nC oC rB pC qC rC sC tC lB uC","420":"I"},Q:{"132":"vC"},R:{"132":"wC"},S:{"164":"xC"}},B:4,C:"CSS3 Multiple column layout"};
