module.exports={A:{A:{"2":"J D E F A B xB"},B:{"1":"P Q R S T U V W Z a b c d e f g h i j k l H X m","2":"C K","132":"L G M N O"},C:{"1":"HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W Z a b c d e f g h i j k l H X m","2":"yB mB I n J D E F A B C K L G M N O o p q r s t u v w x y z zB 0B","132":"0 1 2 3 4 5 6 7 8 9 AB BB CB DB EB FB GB"},D:{"1":"YB ZB aB bB cB dB eB fB gB hB iB P Q R S T U V W Z a b c d e f g h i j k l H X m 1B 2B 3B","2":"0 1 2 3 4 I n J D E F A B C K L G M N O o p q r s t u v w x y z","132":"5 6 7 8 9 AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB"},E:{"1":"A B C K L G 8B rB jB kB 9B AC BC sB tB uB lB CC","2":"I n J D 4B qB 5B 6B","132":"E F 7B"},F:{"1":"OB PB QB RB SB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W","2":"F B C G M N O o p q r DC EC FC GC jB vB HC kB","132":"0 1 2 3 4 5 6 7 8 9 s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB"},G:{"1":"NC OC PC QC RC SC TC UC VC WC XC YC ZC aC bC sB tB uB lB","2":"qB IC wB JC KC LC","16":"E","132":"MC"},H:{"2":"cC"},I:{"1":"H","2":"mB I dC eC fC gC wB hC iC"},J:{"1":"A","2":"D"},K:{"1":"Y","2":"A B C jB vB kB"},L:{"1":"X"},M:{"1":"H"},N:{"2":"A B"},O:{"132":"jC"},P:{"1":"rB pC qC rC sC tC lB uC","132":"I kC lC mC nC oC"},Q:{"132":"vC"},R:{"132":"wC"},S:{"1":"xC"}},B:1,C:"Path2D"};
