module.exports={A:{A:{"2":"xB","8":"J D E","772":"F A B"},B:{"1":"P Q R S T U V W Z a b c d e f g h i j k l H X m","513":"C K L G M N O"},C:{"1":"0 1 2 3 4 5 6 7 8 9 mB I n J D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W Z a b c d e f g h i j k l H X m zB 0B","4":"yB"},D:{"1":"0 1 2 3 4 5 6 7 8 9 I n J D E F A B C K L G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB nB SB oB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R S T U V W Z a b c d e f g h i j k l H X m 1B 2B 3B"},E:{"1":"I n J D E F A B C K L G qB 5B 6B 7B 8B rB jB kB 9B AC BC sB tB uB lB CC","4":"4B"},F:{"1":"0 1 2 3 4 5 6 7 8 9 F B C G M N O o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB QB RB SB TB UB Y VB WB XB YB ZB aB bB cB dB eB fB gB hB iB P Q R pB S T U V W DC EC FC GC jB vB HC kB"},G:{"1":"E qB IC wB JC KC LC MC NC OC PC QC RC SC TC UC VC WC XC YC ZC aC bC sB tB uB lB"},H:{"1":"cC"},I:{"1":"H hC iC","2":"dC eC fC","132":"mB I gC wB"},J:{"1":"D A"},K:{"1":"A B C Y jB vB kB"},L:{"1":"X"},M:{"1":"H"},N:{"257":"A B"},O:{"1":"jC"},P:{"1":"I kC lC mC nC oC rB pC qC rC sC tC lB uC"},Q:{"1":"vC"},R:{"1":"wC"},S:{"1":"xC"}},B:4,C:"SVG (basic support)"};
