import type { FormDataLike } from "../FormDataLike.js";
/**
 * Check if given object is FormData
 *
 * @param value an object to test
 */
export declare const isFormData: (value: unknown) => value is FormDataLike;
